@@ -35,7 +35,9 @@
 #include "gdata-category.h"
 #include "gdata-parsable.h"
 #include "gdata-parser.h"
+#include "gdata-comparable.h"
 
+static void gdata_category_comparable_init (GDataComparableIface *iface);
 static void gdata_category_finalize (GObject *object);
 static void gdata_category_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_category_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -54,8 +56,8 @@ enum {
 	PROP_LABEL
 };
 
-G_DEFINE_TYPE (GDataCategory, gdata_category, GDATA_TYPE_PARSABLE)
-#define GDATA_CATEGORY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_CATEGORY, GDataCategoryPrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataCategory, gdata_category, GDATA_TYPE_PARSABLE,
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_COMPARABLE, gdata_category_comparable_init))
 
 static void
 gdata_category_class_init (GDataCategoryClass *klass)
@@ -79,15 +81,16 @@ gdata_category_class_init (GDataCategoryClass *klass)
 	 * Identifies the category to which the entry or feed belongs.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">Atom specification</ulink>.
+	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">
+	 * Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TERM,
-				g_param_spec_string ("term",
-					"Term", "Identifies the category to which the entry or feed belongs.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("term",
+	                                                      "Term", "Identifies the category to which the entry or feed belongs.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCategory:scheme:
@@ -95,15 +98,16 @@ gdata_category_class_init (GDataCategoryClass *klass)
 	 * An IRI that identifies a categorization scheme.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">Atom specification</ulink>.
+	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">
+	 * Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SCHEME,
-				g_param_spec_string ("scheme",
-					"Scheme", "An IRI that identifies a categorization scheme.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("scheme",
+	                                                      "Scheme", "An IRI that identifies a categorization scheme.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCategory:label:
@@ -111,15 +115,28 @@ gdata_category_class_init (GDataCategoryClass *klass)
 	 * A human-readable label for display in end-user applications.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">Atom specification</ulink>.
+	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">
+	 * Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LABEL,
-				g_param_spec_string ("label",
-					"Label", "A human-readable label for display in end-user applications.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("label",
+	                                                      "Label", "A human-readable label for display in end-user applications.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
+
+static gint
+compare_with (GDataComparable *self, GDataComparable *other)
+{
+	return g_strcmp0 (((GDataCategory*) self)->priv->term, ((GDataCategory*) other)->priv->term);
+}
+
+static void
+gdata_category_comparable_init (GDataComparableIface *iface)
+{
+	iface->compare_with = compare_with;
 }
 
 static void
@@ -221,8 +238,8 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
 /**
  * gdata_category_new:
  * @term: a category identifier
- * @scheme: an IRI to define the categorisation scheme, or %NULL
- * @label: a human-readable label for the category, or %NULL
+ * @scheme: (allow-none): an IRI to define the categorisation scheme, or %NULL
+ * @label: (allow-none): a human-readable label for the category, or %NULL
  *
  * Creates a new #GDataCategory. More information is available in the <ulink type="http"
  * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.category">Atom specification</ulink>.
@@ -236,35 +253,6 @@ gdata_category_new (const gchar *term, const gchar *scheme, const gchar *label)
 	return g_object_new (GDATA_TYPE_CATEGORY, "term", term, "scheme", scheme, "label", label, NULL);
 }
 
-/**
- * gdata_category_compare:
- * @a: a #GDataCategory, or %NULL
- * @b: another #GDataCategory, or %NULL
- *
- * Compares the two categories in a strcmp() fashion. %NULL values are handled gracefully, with
- * <code class="literal">0</code> returned if both @a and @b are %NULL, <code class="literal">-1</code> if @a is %NULL
- * and <code class="literal">1</code> if @b is %NULL.
- *
- * The comparison of non-%NULL values is done on the basis of the @term property of the #GDataCategory<!-- -->s.
- *
- * Return value: <code class="literal">0</code> if @a equals @b, <code class="literal">-1</code> or <code class="literal">1</code> as
- * appropriate otherwise
- *
- * Since: 0.4.0
- **/
-gint
-gdata_category_compare (const GDataCategory *a, const GDataCategory *b)
-{
-	if (a == NULL && b != NULL)
-		return -1;
-	else if (a != NULL && b == NULL)
-		return 1;
-
-	if (a == b)
-		return 0;
-	return g_strcmp0 (a->priv->term, b->priv->term);
-}
-
 /**
  * gdata_category_get_term:
  * @self: a #GDataCategory
@@ -322,7 +310,7 @@ gdata_category_get_scheme (GDataCategory *self)
 /**
  * gdata_category_set_scheme:
  * @self: a #GDataCategory
- * @scheme: the new scheme for the category, or %NULL
+ * @scheme: (allow-none): the new scheme for the category, or %NULL
  *
  * Sets the #GDataCategory:scheme property to @scheme.
  *
@@ -360,7 +348,7 @@ gdata_category_get_label (GDataCategory *self)
 /**
  * gdata_category_set_label:
  * @self: a #GDataCategory
- * @label: the new label for the category, or %NULL
+ * @label: (allow-none): the new label for the category, or %NULL
  *
  * Sets the #GDataCategory:label property to @label.
  *
