@@ -47,6 +47,7 @@
 #include "gdata-documents-spreadsheet.h"
 #include "gdata-documents-text.h"
 #include "gdata-documents-presentation.h"
+#include "gdata-batchable.h"
 #include "gdata-service.h"
 #include "gdata-private.h"
 #include "gdata-upload-stream.h"
@@ -62,7 +63,7 @@ static void gdata_documents_service_get_property (GObject *object, guint propert
 static void notify_authenticated_cb (GObject *service, GParamSpec *pspec, GObject *self);
 static void notify_proxy_uri_cb (GObject *service, GParamSpec *pspec, GObject *self);
 static GDataDocumentsEntry *upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
-						    const gchar *method, const gchar *upload_uri, GCancellable *cancellable, GError **error);
+                                                    const gchar *method, const gchar *upload_uri, GCancellable *cancellable, GError **error);
 
 struct _GDataDocumentsServicePrivate {
 	GDataService *spreadsheet_service;
@@ -72,8 +73,7 @@ enum {
 	PROP_SPREADSHEET_SERVICE = 1
 };
 
-G_DEFINE_TYPE (GDataDocumentsService, gdata_documents_service, GDATA_TYPE_SERVICE)
-#define GDATA_DOCUMENTS_SERVICE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_DOCUMENTS_SERVICE, GDataDocumentsServicePrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataDocumentsService, gdata_documents_service, GDATA_TYPE_SERVICE, G_IMPLEMENT_INTERFACE (GDATA_TYPE_BATCHABLE, NULL))
 
 static void
 gdata_documents_service_class_init (GDataDocumentsServiceClass *klass)
@@ -101,10 +101,10 @@ gdata_documents_service_class_init (GDataDocumentsServiceClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SPREADSHEET_SERVICE,
-				g_param_spec_object ("spreadsheet-service",
-					"Spreadsheet service", "Another service for spreadsheets.",
-					GDATA_TYPE_SERVICE,
-					G_PARAM_READABLE));
+	                                 g_param_spec_object ("spreadsheet-service",
+	                                                      "Spreadsheet service", "Another service for spreadsheets.",
+	                                                      GDATA_TYPE_SERVICE,
+	                                                      G_PARAM_READABLE));
 }
 
 static void
@@ -115,6 +115,34 @@ gdata_documents_service_init (GDataDocumentsService *self)
 	g_signal_connect (self, "notify::proxy-uri", G_CALLBACK (notify_proxy_uri_cb), NULL);
 }
 
+static void
+gdata_documents_service_dispose (GObject *object)
+{
+	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE (object)->priv;
+
+	if (priv->spreadsheet_service != NULL)
+		g_object_unref (priv->spreadsheet_service);
+	priv->spreadsheet_service = NULL;
+
+	G_OBJECT_CLASS (gdata_documents_service_parent_class)->dispose (object);
+}
+
+static void
+gdata_documents_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
+{
+	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE (object)->priv;
+
+	switch (property_id) {
+		case PROP_SPREADSHEET_SERVICE:
+			g_value_set_object (value, priv->spreadsheet_service);
+			break;
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
 /**
  * gdata_documents_service_new:
  * @client_id: your application's client ID
@@ -131,67 +159,45 @@ gdata_documents_service_new (const gchar *client_id)
 	g_return_val_if_fail (client_id != NULL, NULL);
 
 	return g_object_new (GDATA_TYPE_DOCUMENTS_SERVICE,
-			     "client-id", client_id,
-			     NULL);
-}
-
-static void
-gdata_documents_service_dispose (GObject *object)
-{
-	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE_GET_PRIVATE (object);
-
-	if (priv->spreadsheet_service != NULL)
-		g_object_unref (priv->spreadsheet_service);
-	priv->spreadsheet_service = NULL;
-
-	G_OBJECT_CLASS (gdata_documents_service_parent_class)->dispose (object);
-}
-
-static void
-gdata_documents_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
-{
-	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE_GET_PRIVATE (object);
-
-	switch (property_id) {
-		case PROP_SPREADSHEET_SERVICE:
-			g_value_set_object (value, priv->spreadsheet_service);
-			break;
-		default:
-			/* We don't have any other property... */
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-			break;
-	}
+	                     "client-id", client_id,
+	                     NULL);
 }
 
 /**
  * gdata_documents_service_query_documents:
  * @self: a #GDataDocumentsService
- * @query: a #GDataQuery with the query parameters, or %NULL
+ * @query: (allow-none): a #GDataDocumentsQuery with the query parameters, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
+ * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
- * Queries the service to return a list of documents matching the given @query.
+ * Queries the service to return a list of documents matching the given @query. Note that @query has to be a #GDataDocumentsQuery, rather than just
+ * a #GDataQuery, as it uses the folder ID specified in #GDataDocumentsQuery:folder-id.
  *
  * For more details, see gdata_service_query().
  *
- * Return value: a #GDataDocumentsFeed of query results; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataDocumentsFeed of query results; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataDocumentsFeed *
 gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocumentsQuery *query, GCancellable *cancellable,
-					 GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					 GError **error)
+                                         GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                         GError **error)
 {
 	GDataFeed *feed;
 	gchar *request_uri;
 
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_DOCUMENTS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
 	/* Ensure we're authenticated first */
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to query documents."));
+		                     _("You must be authenticated to query documents."));
 		return NULL;
 	}
 
@@ -209,80 +215,14 @@ gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocum
 }
 
 /**
- * gdata_documents_service_query_single_document:
+ * gdata_documents_service_query_documents_async: (skip)
  * @self: a #GDataDocumentsService
- * @document_type: the expected #GType of the queried entry
- * @document_id: the document ID of the queried document
- * @cancellable: a #GCancellable, or %NULL
- * @error: a #GError, or %NULL
- *
- * Retrieves information about a single document with the given document ID.
- *
- * @document_type should be the expected type of the document to be returned. e.g. %GDATA_TYPE_DOCUMENTS_SPREADSHEET if you're querying
- * for a spreadsheet.
- *
- * @document_id should be the ID of the document as returned by gdata_documents_entry_get_document_id().
- *
- * Parameters and errors are as for gdata_service_query().
- *
- * Return value: a #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
- *
- * Since: 0.5.0
- **/
-GDataDocumentsEntry *
-gdata_documents_service_query_single_document (GDataDocumentsService *self, GType document_type, const gchar *document_id,
-					       GCancellable *cancellable, GError **error)
-{
-	GDataDocumentsEntry *document;
-	SoupMessage *message;
-	GDataDocumentsQuery *query;
-	gchar *resource_id, *request_uri;
-
-	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
-	g_return_val_if_fail (document_id != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	if (document_type == GDATA_TYPE_DOCUMENTS_FOLDER)
-		resource_id = g_strconcat ("folder:", document_id, NULL);
-	else if (document_type == GDATA_TYPE_DOCUMENTS_SPREADSHEET)
-		resource_id = g_strconcat ("spreadsheet:", document_id, NULL);
-	else if (document_type == GDATA_TYPE_DOCUMENTS_TEXT)
-		resource_id = g_strconcat ("document:", document_id, NULL);
-	else if (document_type == GDATA_TYPE_DOCUMENTS_PRESENTATION)
-		resource_id = g_strconcat ("presentation:", document_id, NULL);
-	else
-		g_assert_not_reached ();
-
-	query = gdata_documents_query_new (NULL);
-	gdata_query_set_entry_id (GDATA_QUERY (query), resource_id);
-	g_free (resource_id);
-
-	request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
-	message = _gdata_service_query (GDATA_SERVICE (self), request_uri, GDATA_QUERY (query), cancellable, NULL, NULL, error);
-	g_free (request_uri);
-	g_object_unref (query);
-
-	if (message == NULL)
-		return NULL;
-
-	g_assert (message->response_body->data != NULL);
-	document = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (document_type, message->response_body->data,
-								       message->response_body->length, error));
-	g_object_unref (message);
-
-	return document;
-}
-
-/**
- * gdata_documents_service_query_documents_async:
- * @self: a #GDataDocumentsService
- * @query: a #GDataQuery with the query parameters, or %NULL
+ * @query: (allow-none): a #GDataDocumentsQuery with the query parameters, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: data to pass to the @callback function
+ * @user_data: (closure): data to pass to the @callback function
  *
  * Queries the service to return a list of documents matching the given @query. @self and
  * @query are both reffed when this function is called, so can safely be unreffed after this function returns.
@@ -294,16 +234,21 @@ gdata_documents_service_query_single_document (GDataDocumentsService *self, GTyp
  **/
 void
 gdata_documents_service_query_documents_async (GDataDocumentsService *self, GDataDocumentsQuery *query, GCancellable *cancellable,
-					       GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					       GAsyncReadyCallback callback, gpointer user_data)
+                                               GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                               GAsyncReadyCallback callback, gpointer user_data)
 {
 	gchar *request_uri;
 
+	g_return_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self));
+	g_return_if_fail (query == NULL || GDATA_IS_DOCUMENTS_QUERY (query));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
+
 	/* Ensure we're authenticated first */
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
-						     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-						     _("You must be authenticated to query documents."));
+		                                     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+		                                     _("You must be authenticated to query documents."));
 		return;
 	}
 
@@ -321,7 +266,7 @@ static void
 notify_authenticated_cb (GObject *service, GParamSpec *pspec, GObject *self)
 {
 	GDataService *spreadsheet_service;
-	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE_GET_PRIVATE (GDATA_DOCUMENTS_SERVICE (service));
+	GDataDocumentsServicePrivate *priv = GDATA_DOCUMENTS_SERVICE (service)->priv;
 
 	if (priv->spreadsheet_service != NULL)
 		g_object_unref (priv->spreadsheet_service);
@@ -329,7 +274,7 @@ notify_authenticated_cb (GObject *service, GParamSpec *pspec, GObject *self)
 	spreadsheet_service = g_object_new (GDATA_TYPE_SERVICE, "client-id", gdata_service_get_client_id (GDATA_SERVICE (service)), NULL);
 	GDATA_SERVICE_GET_CLASS (spreadsheet_service)->service_name = "wise";
 	gdata_service_authenticate (spreadsheet_service, gdata_service_get_username (GDATA_SERVICE (service)),
-				    gdata_service_get_password (GDATA_SERVICE (service)), NULL, NULL);
+	                            gdata_service_get_password (GDATA_SERVICE (service)), NULL, NULL);
 	priv->spreadsheet_service = spreadsheet_service;
 }
 
@@ -348,7 +293,7 @@ notify_proxy_uri_cb (GObject *service, GParamSpec *pspec, GObject *self)
 
 static GDataDocumentsEntry *
 upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file, const gchar *method,
-			const gchar *upload_uri, GCancellable *cancellable, GError **error)
+                        const gchar *upload_uri, GCancellable *cancellable, GError **error)
 {
 	GDataDocumentsEntry *new_entry;
 	GOutputStream *output_stream;
@@ -356,7 +301,7 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
 	const gchar *slug = NULL, *content_type = NULL, *response_body;
 	gssize response_length;
 	GFileInfo *file_info = NULL;
-	GType new_document_type;
+	GType new_document_type = G_TYPE_INVALID;
 	GError *child_error = NULL;
 
 	/* Get some information about the file we're uploading */
@@ -376,35 +321,36 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
 		 * Bug filed with Google: http://code.google.com/p/gdata-issues/issues/detail?id=1127 */
 		if (strcmp (content_type, "application/vnd.oasis.opendocument.spreadsheet") == 0)
 			content_type = "application/x-vnd.oasis.opendocument.spreadsheet";
+
+		if (document == NULL) {
+			/* Get the document type of the document which is being uploaded */
+			if (strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
+			    strcmp (content_type, "text/tab-separated-values") == 0 ||
+			    strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
+			    strcmp (content_type, "application/vnd.ms-excel") == 0) {
+				new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
+			} else if (strcmp (content_type, "application/msword") == 0 ||
+			           strcmp (content_type, "application/vnd.oasis.opendocument.text") == 0 ||
+			           strcmp (content_type, "application/rtf") == 0 ||
+			           strcmp (content_type, "text/html") == 0 ||
+			           strcmp (content_type, "application/vnd.sun.xml.writer") == 0 ||
+			           strcmp (content_type, "text/plain") == 0) {
+				new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
+			} else if (strcmp (content_type, "application/vnd.ms-powerpoint") == 0) {
+				new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
+			} else {
+				g_set_error_literal (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
+				                     _("The supplied document had an invalid content type."));
+				if (file_info != NULL)
+					g_object_unref (file_info);
+				return NULL;
+			}
+		}
 	}
 
 	/* Determine the type of the document we're uploading */
-	if (document == NULL) {
-		/* We get the document type of the document which is being uploaded */
-		if (strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-		    strcmp (content_type, "text/tab-separated-values") == 0 ||
-		    strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-		    strcmp (content_type, "application/vnd.ms-excel") == 0) {
-			new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
-		} else if (strcmp (content_type, "application/msword") == 0 ||
-			 strcmp (content_type, "application/vnd.oasis.opendocument.text") == 0 ||
-			 strcmp (content_type, "application/rtf") == 0 ||
-			 strcmp (content_type, "text/html") == 0 ||
-			 strcmp (content_type, "application/vnd.sun.xml.writer") == 0 ||
-			 strcmp (content_type, "text/plain") == 0) {
-			new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
-		} else if (strcmp (content_type, "application/vnd.ms-powerpoint") == 0) {
-			new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
-		} else {
-			g_set_error_literal (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
-					     _("The supplied document had an invalid content type."));
-			if (file_info != NULL)
-				g_object_unref (file_info);
-			return NULL;
-		}
-	} else {
+	if (document != NULL)
 		new_document_type = G_OBJECT_TYPE (document);
-	}
 
 	/* We need streaming file I/O: GDataUploadStream */
 	output_stream = gdata_upload_stream_new (GDATA_SERVICE (self), method, upload_uri, GDATA_ENTRY (document), slug, content_type);
@@ -422,7 +368,7 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
 	}
 
 	g_output_stream_splice (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-				cancellable, &child_error);
+	                        cancellable, &child_error);
 
 	g_object_unref (input_stream);
 	if (child_error != NULL) {
@@ -444,9 +390,9 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
 /**
  * gdata_documents_service_upload_document:
  * @self: an authenticated #GDataDocumentsService
- * @document: the #GDataDocumentsEntry to insert, or %NULL
- * @document_file: the document to upload, or %NULL
- * @folder: the folder to which the document should be uploaded, or %NULL
+ * @document: (allow-none): the #GDataDocumentsEntry to insert, or %NULL
+ * @document_file: (allow-none): the document to upload, or %NULL
+ * @folder: (allow-none): the folder to which the document should be uploaded, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -463,13 +409,13 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
  * If there is a problem reading @document_file, an error from g_file_load_contents() or g_file_query_info() will be returned. Other errors from
  * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataDocumentsEntry *
-gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file, GDataDocumentsFolder *folder,
-					 GCancellable *cancellable, GError **error)
+gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
+                                         GDataDocumentsFolder *folder, GCancellable *cancellable, GError **error)
 {
 	GDataDocumentsEntry *new_document;
 	gchar *upload_uri;
@@ -480,16 +426,17 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
 	g_return_val_if_fail (document != NULL || document_file != NULL, NULL);
 	g_return_val_if_fail (folder == NULL || GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to upload documents."));
+		                     _("You must be authenticated to upload documents."));
 		return NULL;
 	}
 
 	if (document != NULL && gdata_entry_is_inserted (GDATA_ENTRY (document)) == TRUE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-				     _("The document has already been uploaded."));
+		                     _("The document has already been uploaded."));
 		return NULL;
 	}
 
@@ -497,7 +444,7 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
 
 	if (document_file == NULL) {
 		new_document = GDATA_DOCUMENTS_ENTRY (gdata_service_insert_entry (GDATA_SERVICE (self), upload_uri, GDATA_ENTRY (document),
-										  cancellable, error));
+		                                                                  cancellable, error));
 	} else {
 		new_document = upload_update_document (self, document, document_file, SOUP_METHOD_POST, upload_uri, cancellable, error);
 	}
@@ -510,7 +457,7 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  * gdata_documents_service_update_document:
  * @self: a #GDataDocumentsService
  * @document: the #GDataDocumentsEntry to update
- * @document_file: the local document file containing the new data, or %NULL
+ * @document_file: (allow-none): the local document file containing the new data, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -522,13 +469,13 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  *
  * For more details, see gdata_service_insert_entry().
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataDocumentsEntry *
 gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
-					 GCancellable *cancellable, GError **error)
+                                         GCancellable *cancellable, GError **error)
 {
 	GDataLink *update_link;
 
@@ -536,10 +483,11 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
 	g_return_val_if_fail (document_file == NULL || G_IS_FILE (document_file), NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to update documents."));
+		                     _("You must be authenticated to update documents."));
 		return NULL;
 	}
 
@@ -565,17 +513,16 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
  *
  * Errors from #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataDocumentsEntry *
 gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GDataDocumentsEntry *document, GDataDocumentsFolder *folder,
-						 GCancellable *cancellable, GError **error)
+                                                 GCancellable *cancellable, GError **error)
 {
-	GDataServiceClass *klass;
 	GDataDocumentsEntry *new_document;
-	gchar *uri, *entry_xml, *upload_data;
+	gchar *uri, *upload_data;
 	const gchar *folder_id;
 	SoupMessage *message;
 	guint status;
@@ -584,54 +531,35 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to move documents."));
+		                     _("You must be authenticated to move documents."));
 		return NULL;
 	}
 
+	/* NOTE: adding a document to a folder doesn't have server-side ETag support (throws "noPostConcurrency" error) */
 	folder_id = gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (folder));
 	g_assert (folder_id != NULL);
 	uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full/folder%3A", folder_id, NULL);
-
-	message = soup_message_new (SOUP_METHOD_POST, uri);
+	message = _gdata_service_build_message (GDATA_SERVICE (self), SOUP_METHOD_POST, uri, NULL, TRUE);
 	g_free (uri);
 
-	/* Make sure subclasses set their headers */
-	klass = GDATA_SERVICE_GET_CLASS (self);
-	if (klass->append_query_headers != NULL)
-		klass->append_query_headers (GDATA_SERVICE (self), message);
-
-	/* Get the XML content */
-	entry_xml = gdata_parsable_get_xml (GDATA_PARSABLE (document));
-
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-		g_object_unref (message);
-		g_free (entry_xml);
-		return NULL;
-	}
-
-	upload_data = g_strconcat ("<?xml version='1.0' encoding='UTF-8'?>", entry_xml, NULL);
-	g_free (entry_xml);
+	/* Append the data */
+	upload_data = gdata_parsable_get_xml (GDATA_PARSABLE (document));
 	soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
 
 	/* Send the message */
-	status = _gdata_service_send_message (GDATA_SERVICE (self), message, error);
-	if (status == SOUP_STATUS_NONE) {
+	status = _gdata_service_send_message (GDATA_SERVICE (self), message, cancellable, error);
+
+	if (status == SOUP_STATUS_NONE || status == SOUP_STATUS_CANCELLED) {
+		/* Redirect error or cancelled */
 		g_object_unref (message);
 		return NULL;
-	}
-
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-		g_object_unref (message);
-		return NULL;
-	}
-
-	if (status != 201) {
+	} else if (status != SOUP_STATUS_CREATED) {
 		/* Error */
+		GDataServiceClass *klass = GDATA_SERVICE_GET_CLASS (self);
 		g_assert (klass->parse_error_response != NULL);
 		klass->parse_error_response (GDATA_SERVICE (self), GDATA_OPERATION_UPDATE, status, message->reason_phrase,
 		                             message->response_body->data, message->response_body->length, error);
@@ -639,10 +567,8 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 		return NULL;
 	}
 
-	/* Build the updated entry */
+	/* Parse the XML; and update the document */
 	g_assert (message->response_body->data != NULL);
-
-	/* Parse the XML; and update the document*/
 	new_document = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (G_OBJECT_TYPE (document), message->response_body->data,
 	                                                                   message->response_body->length, error));
 	g_object_unref (message);
@@ -653,8 +579,8 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 /**
  * gdata_documents_service_remove_document_from_folder:
  * @self: a #GDataDocumentsService
- * @document : the #GDataDocumentsEntry to remove
- * @folder : the #GDataDocumentsFolder from wich we should remove @document
+ * @document: the #GDataDocumentsEntry to remove
+ * @folder: the #GDataDocumentsFolder from which we should remove @document
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -662,16 +588,15 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
  *
  * Errors from #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataDocumentsEntry *
 gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self, GDataDocumentsEntry *document, GDataDocumentsFolder *folder,
-						     GCancellable *cancellable, GError **error)
+                                                     GCancellable *cancellable, GError **error)
 {
 	const gchar *folder_id, *document_id;
-	GDataServiceClass *klass;
 	SoupMessage *message;
 	guint status;
 	gchar *uri;
@@ -680,6 +605,7 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
@@ -694,49 +620,37 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 	g_assert (document_id != NULL);
 
 	if (GDATA_IS_DOCUMENTS_PRESENTATION (document)) {
-		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s",
-		                       _gdata_service_get_scheme (), folder_id, document_id);
+		uri = _gdata_service_build_uri (FALSE, "http://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s",
+		                                folder_id, document_id);
 	} else if (GDATA_IS_DOCUMENTS_SPREADSHEET (document)) {
-		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s",
-		                       _gdata_service_get_scheme (), folder_id, document_id);
+		uri = _gdata_service_build_uri (FALSE, "http://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s",
+		                                folder_id, document_id);
 	} else if (GDATA_IS_DOCUMENTS_TEXT (document)) {
-		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s",
-		                       _gdata_service_get_scheme (), folder_id, document_id);
+		uri = _gdata_service_build_uri (FALSE, "http://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s",
+		                                folder_id, document_id);
 	} else if (GDATA_IS_DOCUMENTS_FOLDER (document)) {
-		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/folder%%3A%s",
-		                       _gdata_service_get_scheme (), folder_id, document_id);
+		uri = _gdata_service_build_uri (FALSE, "http://docs.google.com/feeds/folders/private/full/folder%%3A%s/folder%%3A%s",
+		                                folder_id, document_id);
 	} else {
 		g_assert_not_reached ();
 	}
 
-	message = soup_message_new (SOUP_METHOD_DELETE, uri);
+	message = _gdata_service_build_message (GDATA_SERVICE (self), SOUP_METHOD_DELETE, uri, gdata_entry_get_etag (GDATA_ENTRY (document)), TRUE);
 	g_free (uri);
 
-	/* Make sure subclasses set their headers */
-	klass = GDATA_SERVICE_GET_CLASS (self);
-	if (klass->append_query_headers != NULL)
-		klass->append_query_headers (GDATA_SERVICE (self), message);
-
-	soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (GDATA_ENTRY (document)));
-
 	/* Send the message */
-	status = _gdata_service_send_message (GDATA_SERVICE (self), message, error);
-	if (status == SOUP_STATUS_NONE) {
+	status = _gdata_service_send_message (GDATA_SERVICE (self), message, cancellable, error);
+
+	if (status == SOUP_STATUS_NONE || status == SOUP_STATUS_CANCELLED) {
+		/* Redirect error or cancelled */
 		g_object_unref (message);
 		return NULL;
-	}
-
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-		g_object_unref (message);
-		return NULL;
-	}
-
-	if (status != 200) {
+	} else if (status != SOUP_STATUS_OK) {
 		/* Error */
+		GDataServiceClass *klass = GDATA_SERVICE_GET_CLASS (self);
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (self), GDATA_OPERATION_UPDATE, status, message->reason_phrase, message->response_body->data,
-		                             message->response_body->length, error);
+		klass->parse_error_response (GDATA_SERVICE (self), GDATA_OPERATION_UPDATE, status, message->reason_phrase,
+		                             message->response_body->data, message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
@@ -745,13 +659,13 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 
 	/* Google's servers don't return an updated copy of the entry, so we have to query for it again.
 	 * See: http://code.google.com/p/gdata-issues/issues/detail?id=1380 */
-	return gdata_documents_service_query_single_document (self, G_OBJECT_TYPE (document), gdata_documents_entry_get_document_id (document),
-	                                                      cancellable, error);
+	return GDATA_DOCUMENTS_ENTRY (gdata_service_query_single_entry (GDATA_SERVICE (self), gdata_entry_get_id (GDATA_ENTRY (document)), NULL,
+	                                                                G_OBJECT_TYPE (document), cancellable, error));
 }
 
 /**
  * gdata_documents_service_get_upload_uri:
- * @folder: the #GDataDocumentsFolder into which to upload the document, or %NULL
+ * @folder: (allow-none): the #GDataDocumentsFolder into which to upload the document, or %NULL
  *
  * Gets the upload URI for documents for the service.
  *
