@@ -25,7 +25,6 @@
 
 #define DEVELOPER_KEY "AI39si7Me3Q7zYs6hmkFvpRBD2nrkVjYYsUO5lh_3HdOkGRc9g6Z4nzxZatk_aAo2EsA21k7vrda0OO6oFg2rnhMedZXPyXoEw"
 #define YT_USERNAME "GDataTest"
-#define YT_PASSWORD "gdata"
 
 static void
 test_authentication (void)
@@ -43,15 +42,15 @@ test_authentication (void)
 	g_assert_cmpstr (gdata_youtube_service_get_developer_key (GDATA_YOUTUBE_SERVICE (service)), ==, DEVELOPER_KEY);
 
 	/* Log in */
-	retval = gdata_service_authenticate (service, YT_USERNAME, YT_PASSWORD, NULL, &error);
+	retval = gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (retval == TRUE);
 	g_clear_error (&error);
 
 	/* Check all is as it should be */
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
-	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME "@gmail.com");
-	g_assert_cmpstr (gdata_service_get_password (service), ==, YT_PASSWORD);
+	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
+	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
 	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 
 	g_object_unref (service);
@@ -72,8 +71,8 @@ test_authentication_async_cb (GDataService *service, GAsyncResult *async_result,
 
 	/* Check all is as it should be */
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
-	g_assert_cmpstr (gdata_service_get_username (service), ==, YT_USERNAME "@gmail.com");
-	g_assert_cmpstr (gdata_service_get_password (service), ==, YT_PASSWORD);
+	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
+	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
 	g_assert_cmpstr (gdata_youtube_service_get_youtube_user (GDATA_YOUTUBE_SERVICE (service)), ==, YT_USERNAME);
 }
 
@@ -90,7 +89,7 @@ test_authentication_async (void)
 	g_assert (GDATA_IS_SERVICE (service));
 
 	main_loop = g_main_loop_new (NULL, TRUE);
-	gdata_service_authenticate_async (service, YT_USERNAME, YT_PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, main_loop);
+	gdata_service_authenticate_async (service, USERNAME, PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, main_loop);
 
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
@@ -269,6 +268,7 @@ test_upload_simple (gconstpointer service)
 	GDataMediaCategory *category;
 	GFile *video_file;
 	gchar *xml;
+	const gchar * const tags[] = { "toast", "wedding", NULL };
 	GError *error = NULL;
 
 	video = gdata_youtube_video_new (NULL);
@@ -278,23 +278,25 @@ test_upload_simple (gconstpointer service)
 	category = gdata_media_category_new ("People", "http://gdata.youtube.com/schemas/2007/categories.cat", NULL);
 	gdata_youtube_video_set_category (video, category);
 	g_object_unref (category);
-	gdata_youtube_video_set_keywords (video, "toast, wedding");
+	gdata_youtube_video_set_keywords (video, tags);
 
 	/* Check the XML */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
 	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
 				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
 				"xmlns:app='http://www.w3.org/2007/app'>"
-			 	"<title type='text'>Bad Wedding Toast</title>"
-			 	"<media:group>"
-			 		"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>People</media:category>"
-			 		"<media:title type='plain'>Bad Wedding Toast</media:title>"
-			 		"<media:description type='plain'>I gave a bad toast at my friend&apos;s wedding.</media:description>"
-			 		"<media:keywords>toast, wedding</media:keywords>"
-			 	"</media:group>"
+				"<title type='text'>Bad Wedding Toast</title>"
+				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<media:group>"
+					"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>People</media:category>"
+					"<media:title type='plain'>Bad Wedding Toast</media:title>"
+					"<media:description type='plain'>I gave a bad toast at my friend&apos;s wedding.</media:description>"
+					"<media:keywords>toast,wedding</media:keywords>"
+				"</media:group>"
 				"<app:control>"
 					"<app:draft>no</app:draft>"
 				"</app:control>"
@@ -318,7 +320,7 @@ test_upload_simple (gconstpointer service)
 }
 
 static void
-test_parsing_app_control (gconstpointer service)
+test_parsing_app_control (void)
 {
 	GDataYouTubeVideo *video;
 	GDataYouTubeState *state;
@@ -370,10 +372,10 @@ test_parsing_app_control (gconstpointer service)
 }
 
 static void
-test_parsing_yt_recorded (gconstpointer service)
+test_parsing_yt_recorded (void)
 {
 	GDataYouTubeVideo *video;
-	GTimeVal recorded;
+	gint64 recorded;
 	gchar *xml;
 	GError *error = NULL;
 
@@ -406,17 +408,17 @@ test_parsing_yt_recorded (gconstpointer service)
 	g_clear_error (&error);
 
 	/* Test the recorded date */
-	gdata_youtube_video_get_recorded (video, &recorded);
-	g_assert_cmpint (recorded.tv_sec, ==, 1059868800);
-	g_assert_cmpint (recorded.tv_usec, ==, 0);
+	recorded = gdata_youtube_video_get_recorded (video);
+	g_assert_cmpint (recorded, ==, 1059868800);
 
 	/* Update the state and see if the XML's written out OK */
-	recorded.tv_sec = 1128229200;
-	gdata_youtube_video_set_recorded (video, &recorded);
+	recorded = 1128229200;
+	gdata_youtube_video_set_recorded (video, recorded);
 
 	/* Check the XML */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
 	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -450,6 +452,153 @@ test_parsing_yt_recorded (gconstpointer service)
 	g_object_unref (video);
 }
 
+static void
+test_parsing_yt_access_control (void)
+{
+	GDataYouTubeVideo *video;
+	gchar *xml;
+	GError *error = NULL;
+
+	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
+		"<entry xmlns='http://www.w3.org/2005/Atom' "
+			"xmlns:media='http://search.yahoo.com/mrss/' "
+			"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+			"xmlns:gd='http://schemas.google.com/g/2005' "
+			"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
+			"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+			"<published>2006-05-16T14:06:37.000Z</published>"
+			"<updated>2009-03-23T12:46:58.000Z</updated>"
+			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'/>"
+			"<title>Judas Priest - Painkiller</title>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.youtube.com/watch?v=JAagedeKdcQ'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo'/>"
+			"<author>"
+				"<name>eluves</name>"
+				"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
+			"</author>"
+			"<media:group>"
+				"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
+				"<media:credit role='uploader' scheme='urn:youtube'>eluves</media:credit>"
+				"<media:category label='Music' scheme='http://gdata.youtube.com/schemas/2007/categories.cat'>Music</media:category>"
+			"</media:group>"
+			"<yt:accessControl action='rate' permission='allowed'/>"
+			"<yt:accessControl action='comment' permission='moderated'/>"
+			"<yt:accessControl action='commentVote' permission='denied'/>"
+			"<yt:accessControl action='videoRespond' permission='allowed'/>"
+			"<yt:accessControl action='syndicate' permission='denied'/>"
+			"<yt:accessControl action='random' permission='moderated'/>"
+		"</entry>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_clear_error (&error);
+
+	/* Test the access controls */
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_RATE), ==, GDATA_YOUTUBE_PERMISSION_ALLOWED);
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_COMMENT), ==, GDATA_YOUTUBE_PERMISSION_MODERATED);
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_COMMENT_VOTE), ==, GDATA_YOUTUBE_PERMISSION_DENIED);
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_VIDEO_RESPOND), ==, GDATA_YOUTUBE_PERMISSION_ALLOWED);
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_EMBED), ==, GDATA_YOUTUBE_PERMISSION_DENIED);
+	g_assert_cmpint (gdata_youtube_video_get_access_control (video, GDATA_YOUTUBE_ACTION_SYNDICATE), ==, GDATA_YOUTUBE_PERMISSION_DENIED);
+
+	/* Update some of them and see if the XML's written out OK */
+	gdata_youtube_video_set_access_control (video, GDATA_YOUTUBE_ACTION_RATE, GDATA_YOUTUBE_PERMISSION_MODERATED);
+	gdata_youtube_video_set_access_control (video, GDATA_YOUTUBE_ACTION_EMBED, GDATA_YOUTUBE_PERMISSION_DENIED);
+
+	/* Check the XML */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
+	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
+			 "<entry xmlns='http://www.w3.org/2005/Atom' "
+				"xmlns:media='http://search.yahoo.com/mrss/' "
+				"xmlns:gd='http://schemas.google.com/g/2005' "
+				"xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+				"xmlns:app='http://www.w3.org/2007/app' "
+				"gd:etag='W/\"CEMFSX47eCp7ImA9WxVUGEw.\"'>"
+				"<title type='text'>Judas Priest - Painkiller</title>"
+				"<id>tag:youtube.com,2008:video:JAagedeKdcQ</id>"
+				"<updated>2009-03-23T12:46:58Z</updated>"
+				"<published>2006-05-16T14:06:37Z</published>"
+				"<category term='http://gdata.youtube.com/schemas/2007#video' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<link href='http://www.youtube.com/watch?v=JAagedeKdcQ' rel='http://www.iana.org/assignments/relation/alternate' type='text/html'/>"
+				"<link href='http://gdata.youtube.com/feeds/api/videos/JAagedeKdcQ?client=ytapi-google-jsdemo' rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml'/>"
+				"<author>"
+					"<name>eluves</name>"
+					"<uri>http://gdata.youtube.com/feeds/api/users/eluves</uri>"
+				"</author>"
+				"<media:group>"
+					"<media:category scheme='http://gdata.youtube.com/schemas/2007/categories.cat' label='Music'>Music</media:category>"
+					"<media:title type='plain'>Judas Priest - Painkiller</media:title>"
+				"</media:group>"
+				"<yt:accessControl action='embed' permission='denied'/>"
+				"<yt:accessControl action='random' permission='moderated'/>"
+				"<yt:accessControl action='commentVote' permission='denied'/>"
+				"<yt:accessControl action='rate' permission='moderated'/>"
+				"<yt:accessControl action='comment' permission='moderated'/>"
+				"<yt:accessControl action='syndicate' permission='denied'/>"
+				"<yt:accessControl action='videoRespond' permission='allowed'/>"
+				"<app:control>"
+					"<app:draft>no</app:draft>"
+				"</app:control>"
+			 "</entry>");
+	g_free (xml);
+
+	g_object_unref (video);
+}
+
+static void
+test_parsing_yt_category (void)
+{
+	GDataYouTubeCategory *category;
+	gboolean assignable, deprecated;
+	GError *error = NULL;
+
+	/* Test a non-deprecated category */
+	category = GDATA_YOUTUBE_CATEGORY (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_CATEGORY,
+		"<category xmlns='http://www.w3.org/2005/Atom' xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+			"scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'>"
+			"<yt:assignable/>"
+			"<yt:browsable regions='CZ AU HK'/>"
+		"</category>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_CATEGORY (category));
+	g_clear_error (&error);
+
+	/* Test the category's properties */
+	g_assert (gdata_youtube_category_is_assignable (category) == TRUE);
+	g_assert (gdata_youtube_category_is_browsable (category, "CZ") == TRUE);
+	g_assert (gdata_youtube_category_is_browsable (category, "AU") == TRUE);
+	g_assert (gdata_youtube_category_is_browsable (category, "HK") == TRUE);
+	g_assert (gdata_youtube_category_is_browsable (category, "GB") == FALSE);
+	g_assert (gdata_youtube_category_is_deprecated (category) == FALSE);
+
+	/* Test the properties the other way */
+	g_object_get (category, "is-assignable", &assignable, "is-deprecated", &deprecated, NULL);
+	g_assert (assignable == TRUE);
+	g_assert (deprecated == FALSE);
+
+	g_object_unref (category);
+
+	/* Test a deprecated category */
+	category = GDATA_YOUTUBE_CATEGORY (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_CATEGORY,
+		"<category xmlns='http://www.w3.org/2005/Atom' xmlns:yt='http://gdata.youtube.com/schemas/2007' "
+			"scheme='http://schemas.google.com/g/2005#kind' term='http://gdata.youtube.com/schemas/2007#video'>"
+			"<yt:deprecated/>"
+		"</category>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_CATEGORY (category));
+	g_clear_error (&error);
+
+	/* Test the category's properties */
+	g_assert (gdata_youtube_category_is_assignable (category) == FALSE);
+	g_assert (gdata_youtube_category_is_browsable (category, "CZ") == FALSE);
+	g_assert (gdata_youtube_category_is_browsable (category, "AU") == FALSE);
+	g_assert (gdata_youtube_category_is_browsable (category, "HK") == FALSE);
+	g_assert (gdata_youtube_category_is_browsable (category, "GB") == FALSE);
+	g_assert (gdata_youtube_category_is_deprecated (category) == TRUE);
+
+	g_object_unref (category);
+}
+
 /*static void
 test_parsing_comments_feed_link (void)
 {
@@ -613,7 +762,8 @@ test_query_single (gconstpointer service)
 	GDataYouTubeVideo *video;
 	GError *error = NULL;
 
-	video = gdata_youtube_service_query_single_video (GDATA_YOUTUBE_SERVICE (service), NULL, "_LeQuMpwbW4", NULL, &error);
+	video = GDATA_YOUTUBE_VIDEO (gdata_service_query_single_entry (GDATA_SERVICE (service), "tag:youtube.com,2008:video:_LeQuMpwbW4", NULL,
+	                                                               GDATA_TYPE_YOUTUBE_VIDEO, NULL, &error));
 
 	g_assert_no_error (error);
 	g_assert (video != NULL);
@@ -631,7 +781,7 @@ test_query_single_async_cb (GDataService *service, GAsyncResult *async_result, G
 	GDataYouTubeVideo *video;
 	GError *error = NULL;
 
-	video = gdata_youtube_service_query_single_video_finish (GDATA_YOUTUBE_SERVICE (service), async_result, &error);
+	video = GDATA_YOUTUBE_VIDEO (gdata_service_query_single_entry_finish (GDATA_SERVICE (service), async_result, &error));
 
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
@@ -648,8 +798,8 @@ test_query_single_async (gconstpointer service)
 {
 	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
-	gdata_youtube_service_query_single_video_async (GDATA_YOUTUBE_SERVICE (service), NULL, "_LeQuMpwbW4", NULL,
-							(GAsyncReadyCallback) test_query_single_async_cb, main_loop);
+	gdata_service_query_single_entry_async (GDATA_SERVICE (service), "tag:youtube.com,2008:video:_LeQuMpwbW4", NULL, GDATA_TYPE_YOUTUBE_VIDEO,
+	                                        NULL, (GAsyncReadyCallback) test_query_single_async_cb, main_loop);
 
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
@@ -689,43 +839,285 @@ test_parsing_video_id_from_uri (void)
 	g_assert (video_id == NULL);
 }
 
+static void
+test_categories (gconstpointer service)
+{
+	GDataAPPCategories *app_categories;
+	GList *categories;
+	GError *error = NULL;
+	gchar *category_label, *old_locale;
+
+	app_categories = gdata_youtube_service_get_categories (GDATA_YOUTUBE_SERVICE (service), NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_APP_CATEGORIES (app_categories));
+	g_clear_error (&error);
+
+	categories = gdata_app_categories_get_categories (app_categories);
+	g_assert_cmpint (g_list_length (categories), >, 0);
+	g_assert (GDATA_IS_YOUTUBE_CATEGORY (categories->data));
+
+	/* Save a label for comparison against a different locale */
+	category_label = g_strdup (gdata_category_get_label (GDATA_CATEGORY (categories->data)));
+
+	g_object_unref (app_categories);
+
+	/* Test with a different locale */
+	old_locale = g_strdup (gdata_service_get_locale (GDATA_SERVICE (service)));
+	gdata_service_set_locale (GDATA_SERVICE (service), "it");
+
+	app_categories = gdata_youtube_service_get_categories (GDATA_YOUTUBE_SERVICE (service), NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_APP_CATEGORIES (app_categories));
+	g_clear_error (&error);
+
+	categories = gdata_app_categories_get_categories (app_categories);
+	g_assert_cmpint (g_list_length (categories), >, 0);
+	g_assert (GDATA_IS_YOUTUBE_CATEGORY (categories->data));
+
+	/* Compare the labels */
+	g_assert_cmpstr (category_label, !=, gdata_category_get_label (GDATA_CATEGORY (categories->data)));
+
+	g_object_unref (app_categories);
+	g_free (category_label);
+
+	/* Reset the locale */
+	gdata_service_set_locale (GDATA_SERVICE (service), old_locale);
+	g_free (old_locale);
+}
+
+static void
+test_categories_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GDataAPPCategories *app_categories;
+	GList *categories;
+	GError *error = NULL;
+
+	app_categories = gdata_youtube_service_get_categories_finish (GDATA_YOUTUBE_SERVICE (service), async_result, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_APP_CATEGORIES (app_categories));
+	g_clear_error (&error);
+
+	categories = gdata_app_categories_get_categories (app_categories);
+	g_assert_cmpint (g_list_length (categories), >, 0);
+	g_assert (GDATA_IS_YOUTUBE_CATEGORY (categories->data));
+
+	g_main_loop_quit (main_loop);
+	g_object_unref (app_categories);
+}
+
+static void
+test_categories_async (gconstpointer service)
+{
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_youtube_service_get_categories_async (GDATA_YOUTUBE_SERVICE (service), NULL, (GAsyncReadyCallback) test_categories_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+}
+
+typedef struct {
+	GDataEntry *new_video;
+	GDataEntry *new_video2;
+} BatchData;
+
+static void
+setup_batch (BatchData *data, gconstpointer service)
+{
+	GDataEntry *video;
+	GError *error = NULL;
+
+	/* We can't insert new videos as they'd just hit the moderation queue and cause tests to fail. Instead, we rely on two videos already existing
+	 * on the server with the given IDs. */
+	video = gdata_service_query_single_entry (GDATA_SERVICE (service), "tag:youtube.com,2008:video:RzR2k8yo4NY", NULL, GDATA_TYPE_YOUTUBE_VIDEO,
+	                                          NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+
+	data->new_video = video;
+
+	video = gdata_service_query_single_entry (GDATA_SERVICE (service), "tag:youtube.com,2008:video:VppEcVz8qaI", NULL, GDATA_TYPE_YOUTUBE_VIDEO,
+	                                          NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_YOUTUBE_VIDEO (video));
+
+	data->new_video2 = video;
+}
+
+static void
+test_batch (BatchData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	GDataService *service2;
+	gchar *feed_uri;
+	guint op_id, op_id2;
+	GError *error = NULL;
+
+	/* Here we hardcode the feed URI, but it should really be extracted from a video feed, as the GDATA_LINK_BATCH link.
+	 * It looks like this feed is read-only, so we can only test querying. */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
+
+	/* Check the properties of the operation */
+	g_assert (gdata_batch_operation_get_service (operation) == service);
+	g_assert_cmpstr (gdata_batch_operation_get_feed_uri (operation), ==, "http://gdata.youtube.com/feeds/api/videos/batch");
+
+	g_object_get (operation,
+	              "service", &service2,
+	              "feed-uri", &feed_uri,
+	              NULL);
+
+	g_assert (service2 == service);
+	g_assert_cmpstr (feed_uri, ==, "http://gdata.youtube.com/feeds/api/videos/batch");
+
+	g_object_unref (service2);
+	g_free (feed_uri);
+
+	/* Run a singleton batch operation to query one of the entries */
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL, NULL);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+
+	/* Run another batch operation to query the two entries */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL,
+	                                          NULL);
+	op_id2 = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video2), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video2, NULL,
+	                                           NULL);
+	g_assert_cmpuint (op_id, !=, op_id2);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+}
+
+static void
+test_batch_async_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == TRUE);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async (BatchData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+
+	/* Run an async query operation on the video */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL,
+	                                          NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_batch_operation_run_async (operation, NULL, (GAsyncReadyCallback) test_batch_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+}
+
+static void
+test_batch_async_cancellation_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == FALSE);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async_cancellation (BatchData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+	GCancellable *cancellable;
+
+	/* Run an async query operation on the video */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://gdata.youtube.com/feeds/api/videos/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (data->new_video), GDATA_TYPE_YOUTUBE_VIDEO, data->new_video, NULL,
+	                                          NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+	cancellable = g_cancellable_new ();
+
+	gdata_batch_operation_run_async (operation, cancellable, (GAsyncReadyCallback) test_batch_async_cancellation_cb, main_loop);
+	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+	g_object_unref (cancellable);
+}
+
+static void
+teardown_batch (BatchData *data, gconstpointer service)
+{
+	g_object_unref (data->new_video);
+	g_object_unref (data->new_video2);
+}
+
 int
 main (int argc, char *argv[])
 {
 	gint retval;
-	GDataService *service;
+	GDataService *service = NULL;
 
-	g_type_init ();
-	g_thread_init (NULL);
-	g_test_init (&argc, &argv, NULL);
-	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
+	gdata_test_init (argc, argv);
 
-	service = GDATA_SERVICE (gdata_youtube_service_new (DEVELOPER_KEY, CLIENT_ID));
-	gdata_service_authenticate (service, YT_USERNAME, YT_PASSWORD, NULL, NULL);
+	if (gdata_test_internet () == TRUE) {
+		service = GDATA_SERVICE (gdata_youtube_service_new (DEVELOPER_KEY, CLIENT_ID));
+		gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, NULL);
 
-	g_test_add_func ("/youtube/authentication", test_authentication);
-	if (g_test_thorough () == TRUE)
+		g_test_add_func ("/youtube/authentication", test_authentication);
 		g_test_add_func ("/youtube/authentication_async", test_authentication_async);
-	g_test_add_data_func ("/youtube/query/standard_feed", service, test_query_standard_feed);
-	if (g_test_thorough () == TRUE)
+
+		g_test_add_data_func ("/youtube/query/standard_feed", service, test_query_standard_feed);
 		g_test_add_data_func ("/youtube/query/standard_feed_async", service, test_query_standard_feed_async);
-	g_test_add_data_func ("/youtube/query/related", service, test_query_related);
-	if (g_test_thorough () == TRUE)
+		g_test_add_data_func ("/youtube/query/related", service, test_query_related);
 		g_test_add_data_func ("/youtube/query/related_async", service, test_query_related_async);
-	if (g_test_slow () == TRUE)
+
 		g_test_add_data_func ("/youtube/upload/simple", service, test_upload_simple);
-	g_test_add_data_func ("/youtube/parsing/app:control", service, test_parsing_app_control);
-	/*g_test_add_func ("/youtube/parsing/comments/feedLink", test_parsing_comments_feed_link);*/
-	g_test_add_data_func ("/youtube/parsing/yt:recorded", service, test_parsing_yt_recorded);
-	g_test_add_func ("/youtube/query/uri", test_query_uri);
-	g_test_add_func ("/youtube/query/etag", test_query_etag);
-	g_test_add_data_func ("/youtube/query/single", service, test_query_single);
-	if (g_test_slow () == TRUE)
+
+		g_test_add_data_func ("/youtube/query/single", service, test_query_single);
 		g_test_add_data_func ("/youtube/query/single_async", service, test_query_single_async);
+
+		g_test_add_data_func ("/youtube/categories", service, test_categories);
+		g_test_add_data_func ("/youtube/categories/async", service, test_categories_async);
+
+		g_test_add ("/youtube/batch", BatchData, service, setup_batch, test_batch, teardown_batch);
+		g_test_add ("/youtube/batch/async", BatchData, service, setup_batch, test_batch_async, teardown_batch);
+		g_test_add ("/youtube/batch/async/cancellation", BatchData, service, setup_batch, test_batch_async_cancellation, teardown_batch);
+	}
+
+	g_test_add_func ("/youtube/parsing/app:control", test_parsing_app_control);
+	/*g_test_add_func ("/youtube/parsing/comments/feedLink", test_parsing_comments_feed_link);*/
+	g_test_add_func ("/youtube/parsing/yt:recorded", test_parsing_yt_recorded);
+	g_test_add_func ("/youtube/parsing/yt:accessControl", test_parsing_yt_access_control);
+	g_test_add_func ("/youtube/parsing/yt:category", test_parsing_yt_category);
 	g_test_add_func ("/youtube/parsing/video_id_from_uri", test_parsing_video_id_from_uri);
 
+	g_test_add_func ("/youtube/query/uri", test_query_uri);
+	g_test_add_func ("/youtube/query/etag", test_query_etag);
+
 	retval = g_test_run ();
-	g_object_unref (service);
+
+	if (service != NULL)
+		g_object_unref (service);
 
 	return retval;
 }
